import type { CredentialProtocolMsgReturnType, ServiceAcceptRequestOptions, ServiceOfferCredentialOptions, ServiceRequestCredentialOptions } from '../../CredentialServiceOptions';
import type { ProposeCredentialOptions } from '../../CredentialsModuleOptions';
import type { CredentialFormatService } from '../../formats/CredentialFormatService';
import { CredentialExchangeRecord } from '../../repository/CredentialExchangeRecord';
import { V2IssueCredentialMessage } from './messages/V2IssueCredentialMessage';
import { V2OfferCredentialMessage } from './messages/V2OfferCredentialMessage';
import { V2ProposeCredentialMessage } from './messages/V2ProposeCredentialMessage';
import { V2RequestCredentialMessage } from './messages/V2RequestCredentialMessage';
export interface CreateRequestOptions {
    formatServices: CredentialFormatService[];
    record: CredentialExchangeRecord;
    requestOptions: ServiceRequestCredentialOptions;
    offerMessage: V2OfferCredentialMessage;
    holderDid?: string;
}
export declare class CredentialMessageBuilder {
    /**
     * Create a v2 credential proposal message according to the logic contained in the format service. The format services
     * contain specific logic related to indy, jsonld etc. with others to come.
     *
     * @param formats {@link CredentialFormatService} array of format service objects each containing format-specific logic
     * @param proposal {@link ProposeCredentialOptions} object containing (optionally) the linked attachments
     * @param _threadId optional thread id for this message service
     * @return a version 2.0 credential propose message see {@link V2ProposeCredentialMessage}
     */
    createProposal(formatServices: CredentialFormatService[], proposal: ProposeCredentialOptions): Promise<CredentialProtocolMsgReturnType<V2ProposeCredentialMessage>>;
    /**
     * accept a v2 credential proposal message according to the logic contained in the format service. The format services
     * contain specific logic related to indy, jsonld etc. with others to come.
     *
     * @param message {@link V2ProposeCredentialMessage} object containing (optionally) the linked attachments
     * @param connectionId optional connection id for the agent to agent connection
     * @return a version 2.0 credential record object see {@link CredentialRecord}
     */
    processProposal(message: V2ProposeCredentialMessage, connectionId?: string): CredentialExchangeRecord;
    createOfferAsResponse(formatServices: CredentialFormatService[], credentialRecord: CredentialExchangeRecord, options: ServiceOfferCredentialOptions): Promise<V2OfferCredentialMessage>;
    /**
     * Create a {@link V2RequestCredentialMessage}
     *
     * @param formatService correct service for format, indy, w3c etc.
     * @param record The credential record for which to create the credential request
     * @param offer Additional configuration for the offer if present (might not be for W3C)
     * @returns Object containing request message and associated credential record
     *
     */
    createRequest(options: CreateRequestOptions): Promise<CredentialProtocolMsgReturnType<V2RequestCredentialMessage>>;
    /**
     * Create a {@link V2OfferCredentialMessage} as beginning of protocol process.
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param options attributes of the original offer
     * @returns Object containing offer message and associated credential record
     *
     */
    createOffer(formatServices: CredentialFormatService[], options: ServiceOfferCredentialOptions): Promise<{
        credentialRecord: CredentialExchangeRecord;
        message: V2OfferCredentialMessage;
    }>;
    /**
     * Create a {@link V2IssueCredentialMessage} - we issue the credentials to the holder with this message
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param offerMessage the original offer message
     * @returns Object containing offer message and associated credential record
     *
     */
    createCredential(credentialFormats: CredentialFormatService[], record: CredentialExchangeRecord, serviceOptions: ServiceAcceptRequestOptions, requestMessage: V2RequestCredentialMessage, offerMessage: V2OfferCredentialMessage): Promise<CredentialProtocolMsgReturnType<V2IssueCredentialMessage>>;
    generateId(): string;
}
