"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateIndyMetadata = exports.migrateCredentialRecordToV0_2 = void 0;
const CredentialMetadataTypes_1 = require("../../../../modules/credentials/repository/CredentialMetadataTypes");
const CredentialRepository_1 = require("../../../../modules/credentials/repository/CredentialRepository");
const Metadata_1 = require("../../../Metadata");
/**
 * Migrates the {@link CredentialRecord} to 0.2 compatible format. It fetches all records from storage
 * and applies the needed updates to the records. After a record has been transformed, it is updated
 * in storage and the next record will be transformed.
 *
 * The following transformations are applied:
 *  - {@link updateIndyMetadata}
 */
async function migrateCredentialRecordToV0_2(agent) {
    agent.config.logger.info('Migrating credential records to storage version 0.2');
    const credentialRepository = agent.injectionContainer.resolve(CredentialRepository_1.CredentialRepository);
    agent.config.logger.debug(`Fetching all credential records from storage`);
    const allCredentials = await credentialRepository.getAll();
    agent.config.logger.debug(`Found a total of ${allCredentials.length} credential records to update.`);
    for (const credentialRecord of allCredentials) {
        agent.config.logger.debug(`Migrating credential record with id ${credentialRecord.id} to storage version 0.2`);
        await updateIndyMetadata(agent, credentialRecord);
        await credentialRepository.update(credentialRecord);
        agent.config.logger.debug(`Successfully migrated credential record with id ${credentialRecord.id} to storage version 0.2`);
    }
}
exports.migrateCredentialRecordToV0_2 = migrateCredentialRecordToV0_2;
/**
 * The credential record had a custom `metadata` property in pre-0.1.0 storage that contained the `requestMetadata`, `schemaId` and `credentialDefinition`
 * properties. Later a generic metadata API was added that only allows objects to be stored. Therefore the properties were moved into a different structure.
 *
 * This migration method updates the top level properties to the new nested metadata structure.
 *
 * The following pre-0.1.0 structure:
 *
 * ```json
 * {
 *   "requestMetadata": "<value of requestMetadata>",
 *   "schemaId": "<value of schemaId>",
 *   "credentialDefinitionId": "<value of credential definition id>"
 * }
 * ```
 *
 * Will be transformed into the following 0.2.0 structure:
 *
 * ```json
 * {
 *   "_internal/indyRequest": <value of requestMetadata>,
 *   "_internal/indyCredential": {
 *     "schemaId": "<value of schemaId>",
 *     "credentialDefinitionId": "<value of credential definition id>"
 *   }
 * }
 * ```
 */
async function updateIndyMetadata(agent, credentialRecord) {
    agent.config.logger.debug(`Updating indy metadata to use the generic metadata api available to records.`);
    const _a = credentialRecord.metadata.data, { requestMetadata, schemaId, credentialDefinitionId } = _a, rest = __rest(_a, ["requestMetadata", "schemaId", "credentialDefinitionId"]);
    const metadata = new Metadata_1.Metadata(rest);
    if (requestMetadata) {
        agent.config.logger.trace(`Found top-level 'requestMetadata' key, moving to '${CredentialMetadataTypes_1.CredentialMetadataKeys.IndyRequest}'`);
        metadata.add(CredentialMetadataTypes_1.CredentialMetadataKeys.IndyRequest, Object.assign({}, requestMetadata));
    }
    if (schemaId && typeof schemaId === 'string') {
        agent.config.logger.trace(`Found top-level 'schemaId' key, moving to '${CredentialMetadataTypes_1.CredentialMetadataKeys.IndyCredential}.schemaId'`);
        metadata.add(CredentialMetadataTypes_1.CredentialMetadataKeys.IndyCredential, { schemaId });
    }
    if (credentialDefinitionId && typeof credentialDefinitionId === 'string') {
        agent.config.logger.trace(`Found top-level 'credentialDefinitionId' key, moving to '${CredentialMetadataTypes_1.CredentialMetadataKeys.IndyCredential}.credentialDefinitionId'`);
        metadata.add(CredentialMetadataTypes_1.CredentialMetadataKeys.IndyCredential, { credentialDefinitionId });
    }
    credentialRecord.metadata = metadata;
}
exports.updateIndyMetadata = updateIndyMetadata;
//# sourceMappingURL=credential.js.map